/*
 * QEstEidCommon
 *
 * Copyright (C) 2011 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2011 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "FileDialog.h"

#include "Common.h"

#include <QDesktopServices>

FileDialog::FileDialog( QWidget *parent )
:	QFileDialog( parent )
{
}

FileDialog::Options FileDialog::addOptions()
{
	Common *a = qobject_cast<Common*>(QCoreApplication::instance());
	if( a && a->noNativeFileDialog() )
		return DontUseNativeDialog;
	return 0;
}

QString FileDialog::getDir( const QString &dir )
{
	if( !dir.isEmpty() )
		return dir;
	Common *a = qobject_cast<Common*>(QCoreApplication::instance());
	if( a && !a->lastPath().isEmpty() )
		return a->lastPath();
	return QDesktopServices::storageLocation( QDesktopServices::DocumentsLocation );
}

QString FileDialog::getOpenFileName( QWidget *parent, const QString &caption,
	const QString &dir, const QString &filter, QString *selectedFilter, Options options )
{
	return result( QFileDialog::getOpenFileName( parent,
		caption, getDir( dir ), filter, selectedFilter, options|addOptions() ) );
}

QStringList FileDialog::getOpenFileNames( QWidget *parent, const QString &caption,
	const QString &dir, const QString &filter, QString *selectedFilter, Options options )
{
	return result( QFileDialog::getOpenFileNames( parent,
		caption, getDir( dir ), filter, selectedFilter,
#ifdef Q_OS_WIN
		DontResolveSymlinks|options|addOptions() ) );
#else
		options|addOptions() ) );
#endif
}

QString FileDialog::getExistingDirectory( QWidget *parent, const QString &caption,
	const QString &dir, Options options )
{
	return result( QFileDialog::getExistingDirectory( parent,
		caption, getDir( dir ), options|addOptions() ) );
}

QString FileDialog::getSaveFileName( QWidget *parent, const QString &caption,
	const QString &dir, const QString &filter, QString *selectedFilter, Options options )
{
	return result( QFileDialog::getSaveFileName( parent,
		caption, getDir( dir ), filter, selectedFilter, options|addOptions() ) );
}

QString FileDialog::result( const QString &str )
{
#if defined(Q_OS_MAC) && defined(QT_MAC_USE_COCOA)
	QString r = str.normalized( QString::NormalizationForm_C );
#else
	QString r = str;
#endif
	Common *a = qobject_cast<Common*>(QCoreApplication::instance());
	if( a && !r.isEmpty() )
		a->setLastPath( QFileInfo( r ).absolutePath() );
	return r;
}

QStringList FileDialog::result( const QStringList &list )
{
	QStringList l;
	Q_FOREACH( const QString &str, list )
		l << result( str );
	return l;
}

